##############################################################################
#
# Copyright (c) 2010 ViFiB SARL and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################


import hashlib
import os
import posixpath
import re
from download import check_md5sum
from slapos.tool.libnetworkcache import NetworkcacheClient


_md5_re = re.compile(r'md5=([a-f0-9]+)')


def _get_md5_from_url(url):
  match = _md5_re.search(url)
  if match:
    return match.group(1)
  return None


def _update_network_cached_log(path, url):
    """Update the networkcached.log file.

    Adding the last key and URL which has been retrieved from networkcached server.

    """
    f = open(path, 'r')
    try:
      key = hashlib.sha512(f.read()).hexdigest()
    finally:
      f.close()

    # Keep the log
    map_path = os.path.join(os.environ.get('PWD'), '.networkcached.log')
    mode = 'r+'
    if not os.path.exists(map_path):
        mode = 'w+'
    f = open(map_path, mode)
    try:
        flag = False
        for line in f:
            if url in line:
                flag = True
                break
        if not flag:
          f.seek(0, 2)
          f.write('%s %s\n' % (key, url))
          f.truncate(f.tell())
    finally:
        f.close()


def download_network_cached(network_cache, path, url, logger, md5sum=None):
    """Download from a network cache provider

    If something fail (providor be offline, or hash_string fail), we ignore
    network cached files.

    return True if download succeeded.
    """
    if network_cache in (None, '',):
        # Not able to use network cache
        return False
    if md5sum is None:
        print "URL ", url
        md5sum = _get_md5_from_url(url) 

    # XXX (lucas): This will be removed.
    key = hashlib.sha512(url).hexdigest()

    network_cached_url = os.path.join(network_cache, key)
    logger.info('Downloading from network cache %s' % network_cached_url)
    try:
        nc = NetworkcacheClient(network_cache)
        result = nc.get(key)

        if result.status != 200:
          logger.info('Fail to download from network cache: File Not Found.')
          return False

        file_content = result.read()
        # save the file
        f = open(path, 'w+b')
        try:
          f.write(file_content)
        finally:
          f.close()

        if not check_md5sum(path, md5sum):
            logger.info('MD5 checksum mismatch downloading %r' % \
                                                           network_cached_url)
            return False
    except IOError, e:
        logger.info('Fail to download from network cache %s: %s' % \
                                                  (network_cached_url, str(e)))
        return False

    return True


def upload_network_cached(network_cache, external_url, path, logger):
    """Upload file to a network cache server"""
    if network_cache in [None, '']:
        logger.debug(
         'Upload cache ignored, network-cache was not provided')
        return False

    try:
        f = open(path, 'r')
        try:
          file_content = f.read()
        finally:
          f.close()

        nc = NetworkcacheClient(network_cache)
        result = nc.put(file_content)
        if result.status == 200:
            _update_network_cached_log(path, external_url)

    except (IOError, EOFError), e:
        logger.info('Fail to upload cache on %s. %s' % \
                                                  (external_url, str(e)))
    return True


def get_filename_from_url(url):
    """Inspired how pip get filename from url.
    """
    url = url.split('#', 1)[0]
    url = url.split('?', 1)[0]
    url = url.rstrip('/')
    name = posixpath.basename(url)
    assert name, (
           'URL %r produced no filename' % url)
    return name
